from django.contrib import admin

from .widgets import AdminMarkdownxWidget
from .models import MarkdownxField


class MarkdownxModelAdmin(admin.ModelAdmin):
    """
    
    """

    formfield_overrides = {
        MarkdownxField: {'widget': AdminMarkdownxWidget}
    }



from django.utils.translation import ugettext_lazy as _
from django.forms import ValidationError


class MarkdownxImageUploadError(ValidationError):
    """

    """

    @staticmethod
    def not_uploaded():
        """
        No file is available to upload.

        :return:
        :rtype:
        """
        return MarkdownxImageUploadError(_('No files have been uploaded.'))

    @staticmethod
    def unsupported_format():
        """
        The file is of a format not defined in :guilabel:`settings.py`
        or if default, in :guilabel:`markdownx/settings.py`.

        :return:
        :rtype:
        """
        return MarkdownxImageUploadError(_('File type is not supported.'))

    @staticmethod
    def invalid_size(current, expected):
        """
        The file is larger in size that the maximum allow in :guilabel:`settings.py` (or the default).

        :param current:
        :type current:
        :param expected:
        :type expected:
        :return:
        :rtype:
        """
        from django.template.defaultfilters import filesizeformat

        return MarkdownxImageUploadError(
            _('Please keep file size under %(max)s. Current file size: %(current)s.') % {
                'max': filesizeformat(expected),
                'current': filesizeformat(current)
            }
        )



from django import VERSION as DJANGO_VERSION
from django import forms
from django.template.loader import get_template
from django.contrib.admin import widgets
from django.conf import settings

from .settings import (
    MARKDOWNX_EDITOR_RESIZABLE,
    MARKDOWNX_URLS_PATH,
    MARKDOWNX_UPLOAD_URLS_PATH,
    MARKDOWNX_SERVER_CALL_LATENCY
)


DEBUG = getattr(settings, 'DEBUG', False)


class MarkdownxWidget(forms.Textarea):
    """

    """

    if DJANGO_VERSION[:2] >= (1, 11):
        template_name = 'markdownx/widget2.html'

    def get_context(self, name, value, attrs=None):
        """

        :param name:
        :type name:
        :param value:
        :type value:
        :param attrs:
        :type attrs:
        :return:
        :rtype:
        """
        if not DJANGO_VERSION[:2] >= (1, 11):
            return super(MarkdownxWidget, self).get_context(name, value, attrs)

        if attrs is None:
            attrs = {}

        attrs.update(self.add_markdownx_attrs(attrs))

        return super(MarkdownxWidget, self).get_context(name, value, attrs)

    def render(self, name, value, attrs=None, renderer=None):
        """

        :param name:
        :type name:
        :param value:
        :type value:
        :param attrs:
        :type attrs:
        :param renderer:
        :type renderer:
        :return:
        :rtype:
        """
        if not DJANGO_VERSION[:2] < (1, 11):
            return super(MarkdownxWidget, self).render(name, value, attrs, renderer)

        attrs = self.build_attrs(attrs, name=name)
        attrs.update(self.add_markdownx_attrs(attrs))

        widget = super(MarkdownxWidget, self).render(name, value, attrs)

        template = get_template('markdownx/widget.html')

        return template.render({
            'markdownx_editor': widget,
        })

    @staticmethod
    def add_markdownx_attrs(attrs):
        """

        :param attrs:
        :type attrs:
        :return:
        :rtype:
        """
        if 'class' in attrs:
            attrs['class'] += ' markdownx-editor'
        else:
            attrs.update({
                'class': 'markdownx-editor'
            })

        attrs.update({
            'data-markdownx-editor-resizable': MARKDOWNX_EDITOR_RESIZABLE,
            'data-markdownx-urls-path': MARKDOWNX_URLS_PATH,
            'data-markdownx-upload-urls-path': MARKDOWNX_UPLOAD_URLS_PATH,
            'data-markdownx-latency': MARKDOWNX_SERVER_CALL_LATENCY
        })

        return attrs

    class Media:
        """

        """

        js = {
            'markdownx/js/' + ('markdownx.min.js' if not DEBUG else 'markdownx.js'),
        }


class AdminMarkdownxWidget(MarkdownxWidget, widgets.AdminTextareaWidget):
    """

    """

    class Media:
        """

        """

        css = {
            'all': {'markdownx/admin/css/markdownx.css', }
        }

        js = {
            'markdownx/js/' + ('markdownx.min.js' if not DEBUG else 'markdownx.js'),
        }



from django import forms

from .widgets import MarkdownxWidget


class MarkdownxFormField(forms.CharField):
    """

    """

    def __init__(self, *args, **kwargs):
        """

        :param args:
        :type args:
        :param kwargs:
        :type kwargs:
        """
        super(MarkdownxFormField, self).__init__(*args, **kwargs)

        if issubclass(self.widget.__class__, forms.widgets.MultiWidget):
            is_markdownx_widget = any(
                issubclass(item.__class__, MarkdownxWidget)
                for item in getattr(self.widget, 'widgets', list())
            )

            if not is_markdownx_widget:
                self.widget = MarkdownxWidget()

        elif not issubclass(self.widget.__class__, MarkdownxWidget):
            self.widget = MarkdownxWidget()
